<?php

/*
 *       _____ _____ _____                _       _
 *      |_   _/  __ \_   _|              (_)     | |
 *        | | | /  \/ | |  ___  ___   ___ _  __ _| |
 *        | | ||      | | / __|/ _ \ / __| |/ _` | |
 *       _| |_| \__/\ | |_\__ \ (_) | (__| | (_| | |
 *      |_____\_____/ |_(_)___/\___/ \___|_|\__,_|_|
 *                   ___
 *                  |  _|___ ___ ___
 *                  |  _|  _| -_| -_|  LICENCE
 *                  |_| |_| |___|___|
 *
 * IT NEWS  <>  PROGRAMMING  <>  HW & SW  <>  COMMUNITY
 *
 * This source code is part of online courses at IT social
 * network WWW.ICT.SOCIAL
 *
 * Feel free to use it for whatever you want, modify it and share it but
 * don't forget to keep this link in code.
 *
 * For more information visit http://www.ict.social/licences
 * 
 * A simple database wrapper over the PDO class
 */
class Db
{
	/**
	 * @var PDO A database connection
	 */
	private static $connection;

	/**
	 * @var array The default driver settings
	 */
	private static $options = array(
		PDO::ATTR_ERRMODE => PDO::ERRMODE_WARNING,
		PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES utf8",
		PDO::ATTR_EMULATE_PREPARES => false,
	);

	/**
	 * Connects to the database using given credentials
	 * @param string $host Host name
	 * @param string $database Database name
	 * @param string $user Username
	 * @param string $password Password
	 */
	public static function connect($host, $database, $user, $password)
	{
		if (!isset(self::$connection)) {
			$dsn = "mysql:host=$host;dbname=$database";
			self::$connection = new PDO($dsn, $user, $password, self::$options);
		}
	}

	/**
	 * Executes a query and returns the PDO statement
	 * @param array $params An array where the first item represents the query and the other items are its parameters.
	 * @return \PDOStatement PDO statement
	 */
	private static function executeStatement($params)
	{
		$query = array_shift($params);
		$statement = self::$connection->prepare($query);
		$statement->execute($params);
		return $statement;
	}

	/**
	 * Executes a query and returns the number of affected rows. 
	 * Any other parameters will be passed into the query.
	 * @param string $query The query
	 * @return int The number of affected rows
	 */
	public static function query($query) {
		$statement = self::executeStatement(func_get_args());
		return $statement->rowCount();
	}

	/**
	 * Executes a query and returns the value of the first column of the first row.
	 * Any other parameters will be passed into the query.
	 * @param string $query The query
	 * @return mixed The value of the first column of the first row
	 */
	public static function querySingle($query) {
		$statement = self::executeStatement(func_get_args());
		$data = $statement->fetch();
		return $data[0];
	}

	/**
	 * Executes a query and returns the first row of the result. 
	 * Any other parameters will be passed into the query.
	 * @param string $query The query
	 * @return mixed An associative array representing the row or false in no data returned
	 */
	public static function queryOne($query) {
		$statement = self::executeStatement(func_get_args());
		return $statement->fetch(PDO::FETCH_ASSOC);
	}

	/**
	 * Executes a query and returns all resulting rows as an array of associative arrays.
	 * Any other parameters will be passed into the query.
	 * @param string $query The query
	 * @return mixed An array of associative arrays or false in no data returned
	 */
	public static function queryAll($query) {
		$statement = self::executeStatement(func_get_args());
		return $statement->fetchAll(PDO::FETCH_ASSOC);
	}

	/**
	 * Inserts data from an associative array into the database as a new row
	 * @param string $table The table name
	 * @param array $data The associative array where keys preresent columns and values their values
	 * @return int The number of affected rows
	 */
	public static function insert($table, $data) {
		$keys = array_keys($data);
		self::checkIdentifiers(array($table) + $keys);
		$query = "
			INSERT INTO `$table` (`" . implode('`, `', $keys) . "`)
			VALUES (" . str_repeat('?,', count($data) - 1) . "?)
		";
		$params = array_merge(array($query), array_values($data));
		$statement = self::executeStatement($params);
		return $statement->rowCount();
	}

	/**
	 * Executes an update and passes data from an associative array to it
	 * @param string $table The table name
	 * @param array $data The associative array where keys preresent columns and values their values
	 * @param string $condition A string containing the condition (WHERE)
	 * @return mixed The number of affected rows
	 */
	public static function update($table, $data, $condition) {
		$keys = array_keys($data);
		self::checkIdentifiers(array($table) + $keys);
		$query = "
			UPDATE `$table` SET `".
			implode('` = ?, `', array_keys($data)) . "` = ?
			$condition
		";
		$params = array_merge(array($query), array_values($data), array_slice(func_get_args(), 3));
		$statement = self::executeStatement($params);
		return $statement->rowCount();
	}

	/**
	 * Returns the ID of the last inserted row
	 * @return mixed The ID of the last inserted row
	 */
	public static function getLastId()
	{
		return self::$connection->lastInsertId();
	}

	/**
	 * Quotes a given string to protect it against the SQL injection
	 * @param string $string The string
	 * @return mixed The quoted string
	 */
	public static function quote($string)
	{
		return self::$connection->quote($string);
	}

	/**
	 * Checks whether given identifiers don't contain dangerous characters
	 * @param array $identifiers The identifiers
	 * @throws \Exception
	 */
	private static function checkIdentifiers($identifiers)
	{
		foreach ($identifiers as $identifier)
		{
			if (!preg_match('/^[a-zA-Z0-9\_\-]+$/u', $identifier))
				throw new Exception('Dangerous identifier in SQL query');
		}
	}
}
